
/*
 * GDB Stub xfer-spaces support code.
 *
 * Copyright (c) 2012 Alex Rozenman <alex_rozenman@mentor.com>
 * Copyright (c) 2012 Mentor Graphics Corp.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#pragma once

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct Debuggable {

  /* This function shall return the name of the object. The name is 
     considered to be a hierarchical separated by dots.  */
  const char* (*get_name)(void* opaque);

  /* This function shall return the data size (bytes) of 
     this object.  */
  unsigned (*get_data_size)(void* opaque);

  /* Read the data contained in the object. Return the number of
     successfully read bytes.  */
  unsigned (*read_data)(void* opaque, unsigned char* buf,
                        unsigned buf_len);

  /* Write the data into the object. Return the number of successfully
     written bytes.  */
  unsigned (*write_data)(void* opaque, const unsigned char* buf,
                         unsigned buf_len);

} Debuggable;

extern void xfer_spaces_declare_object(Debuggable* debuggable, void* opaque);
extern void xfer_spaces_elaborate();
extern void xfer_spaces_print_tree();
extern void xfer_spaces_dump_xml(FILE* out, const char* annex, const char* conv);
extern unsigned xfer_spaces_read_object(unsigned offset, unsigned len,
                                        unsigned char* data);
extern unsigned xfer_spaces_write_object(unsigned offset, unsigned len,
                                         const unsigned char* data);

#ifdef __cplusplus
}
#endif
